==> tuyaos_demo_ipc_1.0.0.tar.gz: tuyaos_demo_ipc/src/tuya_ipc_media_wave_parse.c <==
/*********************************************************************************
  *Copyright(C),2015-2021, 
  *TUYA 
  *www.tuya.com
  *FileName:    tuya_ipc_media_wave_parse.c
**********************************************************************************/
#include "tuya_ipc_common_demo.h"
#if defined(DEMO_IPC_VOICE_ENABLE) && (DEMO_IPC_VOICE_ENABLE == 1)
#include <string.h>
#include <stdlib.h>
#include "tuya_ipc_media_wave_parse.h"

ty_media_wav_s *ty_media_wav_open(CHAR_T *file_name)
{
    ty_media_wav_s *wav = NULL;
    ty_wav_id_header_s id_header;
    CHAR_T id_str[5] = {0};
    INT_T read_len = 0;

    if (NULL == file_name)
    {
        PR_ERR("file_name is NULL");
        return NULL;
    }

    wav = (ty_media_wav_s *)malloc(sizeof(ty_media_wav_s));
    if (NULL == wav)
    {
        PR_ERR("malloc wav failed");
        return NULL;
    }

    memset(wav, 0, sizeof(ty_media_wav_s));

    wav->fp = fopen(file_name, "r");
    if (NULL == wav->fp)
    {
        PR_ERR("fopen %s failed", file_name);
        free(wav);
        return NULL;
    }

    /* handle RIFF WAVE chunk */
    read_len = fread(&wav->riff, 1, sizeof(ty_wav_riff_s), wav->fp);
    if (read_len < sizeof(ty_wav_riff_s))
    {
        PR_ERR("error wav file: read RIFF chunk fail");
        ty_media_wav_close(&wav);
        return NULL;
    }

    if (strncasecmp("RIFF", wav->riff.id, strlen("RIFF")))
    {
        PR_ERR("no riff, error wav file");
        ty_media_wav_close(&wav);
        return NULL;
    }

    if (strncasecmp("WAVE", wav->riff.type, strlen("WAVE")))
    {
        PR_ERR("no wave, error wav file");
        ty_media_wav_close(&wav);
        return NULL;
    }

    wav->file_size = sizeof(ty_wav_id_header_s) + wav->riff.size;
    wav->data_offset = sizeof(ty_wav_riff_s);

    while (1)
    {
        read_len = fread(&id_header, 1, sizeof(ty_wav_id_header_s), wav->fp);
        if (read_len < sizeof(ty_wav_id_header_s))
        {
            PR_ERR("error wav file: read id header fail");
            ty_media_wav_close(&wav);
            return NULL;
        }

        if (0 == strncasecmp("FMT", id_header.id, strlen("FMT")))
        {
            if (id_header.size != sizeof(ty_wav_format_s) - sizeof(ty_wav_id_header_s))
            {
                PR_ERR("error wav file: wrong FMT size %d", id_header.size);
                ty_media_wav_close(&wav);
                return NULL;
            }
            read_len = fread(&wav->format.compression_code, 1, id_header.size, wav->fp);
            if (read_len < id_header.size)
            {
                PR_ERR("err wav file: read FMT fail");
                ty_media_wav_close(&wav);
                return NULL;
            }
            memcpy(wav->format.id, id_header.id, sizeof(wav->format.id));
            wav->format.size = id_header.size;
            wav->data_offset += sizeof(ty_wav_format_s);
        }
        else if (0 == strncasecmp("DATA", id_header.id, strlen("DATA")))
        {
            memcpy(wav->data.id, id_header.id, sizeof(wav->data.id));
            wav->data.size = id_header.size;
            wav->data_offset += sizeof(ty_wav_id_header_s);
            wav->data_size = id_header.size;
            break;
        }
        else
        {
            memcpy(id_str, id_header.id, sizeof(id_header.id));
            PR_DEBUG("unhandled chunk: %s, size: %d", id_str, id_header.size);
            wav->data_offset += sizeof(ty_wav_id_header_s) + id_header.size;
            fseek(wav->fp, id_header.size, SEEK_CUR);
        }
    }

    return wav;
}

VOID ty_media_wav_close(ty_media_wav_s **wav)
{
    ty_media_wav_s *tmp_wav = NULL;

    if (NULL == wav)
    {
        return;
    }

    tmp_wav = *wav;
    if (NULL == tmp_wav)
    {
        return;
    }

    if (NULL != tmp_wav->fp)
    {
        fclose(tmp_wav->fp);
    }

    free(tmp_wav);
    *wav = NULL;

    return;
}

VOID ty_media_wav_dump(ty_media_wav_s *wav)
{
    CHAR_T str[5] = {0};

    printf("file length: %d\n", wav->file_size);

    printf("\nRIFF WAVE Chunk\n");
    memcpy(str, wav->riff.id, sizeof(wav->riff.id));
    printf("id: %s\n", str);
    printf("size: %d\n", wav->riff.size);
    memcpy(str, wav->riff.type, sizeof(wav->riff.type));
    printf("type: %s\n", str);

    printf("\nFORMAT Chunk\n");
    memcpy(str, wav->format.id, sizeof(wav->format.id));
    printf("id: %s\n", str);
    printf("formattag: 0x%02x\n", wav->format.compression_code);
    printf("channels: %d\n", wav->format.channels);
    printf("samples_per_sec: %d\n", wav->format.samples_per_sec);
    printf("avg_bytes_per_sec: %d\n", wav->format.avg_bytes_per_sec);
    printf("block_align: %d\n", wav->format.block_align);
    printf("bits_per_sample: %d\n", wav->format.bits_per_sample);

    printf("\nDATA Chunk\n");
    memcpy(str, wav->data.id, sizeof(wav->data.id));
    printf("id: %s\n", str);
    printf("size: %d\n", wav->data.size);
    printf("data offset: %d\n", wav->data_offset);

    return;
}

INT_T ty_media_wav_check(CHAR_T *file_name)
{
    ty_media_wav_s *pwav = NULL;
    UINT_T file_len = 0;

    pwav = ty_media_wav_open(file_name);
    if (pwav == NULL)
    {
        PR_ERR("open media wav %s failed", file_name);
        return -1;
    }

    /* get file length */
    fseek(pwav->fp, 0, SEEK_END);
    file_len = ftell(pwav->fp);

    if (file_len < pwav->data_offset + pwav->data_size)
    {
        PR_ERR("file_len = %d, expected len = %d\n", file_len, pwav->data_offset + pwav->data_size);
        ty_media_wav_close(&pwav);
        return -1;
    }

    ty_media_wav_close(&pwav);
    return 0;
}
#endif

==> tuyaos_demo_ipc_1.0.3.tar.gz: tuyaos_demo_ipc/src/tuya_ipc_media_wave_parse.c <==
/*********************************************************************************
  *Copyright(C),2015-2021, 
  *TUYA 
  *www.tuya.com
  *FileName:    tuya_ipc_media_wave_parse.c
**********************************************************************************/
#include "tuya_ipc_common_demo.h"
#if defined(DEMO_IPC_VOICE_ENABLE) && (DEMO_IPC_VOICE_ENABLE == 1)
#include <string.h>
#include <stdlib.h>
#include "tuya_ipc_media_wave_parse.h"

ty_media_wav_s *ty_media_wav_open(CHAR_T *file_name)
{
    ty_media_wav_s *wav = NULL;
    ty_wav_id_header_s id_header;
    CHAR_T id_str[5] = {0};
    INT_T read_len = 0;

    if (NULL == file_name)
    {
        PR_ERR("file_name is NULL");
        return NULL;
    }

    wav = (ty_media_wav_s *)malloc(sizeof(ty_media_wav_s));
    if (NULL == wav)
    {
        PR_ERR("malloc wav failed");
        return NULL;
    }

    memset(wav, 0, sizeof(ty_media_wav_s));

    wav->fp = fopen(file_name, "r");
    if (NULL == wav->fp)
    {
        PR_ERR("fopen %s failed", file_name);
        free(wav);
        return NULL;
    }

    /* handle RIFF WAVE chunk */
    read_len = fread(&wav->riff, 1, sizeof(ty_wav_riff_s), wav->fp);
    if (read_len < sizeof(ty_wav_riff_s))
    {
        PR_ERR("error wav file: read RIFF chunk fail");
        ty_media_wav_close(&wav);
        return NULL;
    }

    if (strncasecmp("RIFF", wav->riff.id, strlen("RIFF")))
    {
        PR_ERR("no riff, error wav file");
        ty_media_wav_close(&wav);
        return NULL;
    }

    if (strncasecmp("WAVE", wav->riff.type, strlen("WAVE")))
    {
        PR_ERR("no wave, error wav file");
        ty_media_wav_close(&wav);
        return NULL;
    }

    wav->file_size = sizeof(ty_wav_id_header_s) + wav->riff.size;
    wav->data_offset = sizeof(ty_wav_riff_s);

    while (1)
    {
        read_len = fread(&id_header, 1, sizeof(ty_wav_id_header_s), wav->fp);
        if (read_len < sizeof(ty_wav_id_header_s))
        {
            PR_ERR("error wav file: read id header fail");
            ty_media_wav_close(&wav);
            return NULL;
        }

        if (0 == strncasecmp("FMT", id_header.id, strlen("FMT")))
        {
            if (id_header.size != sizeof(ty_wav_format_s) - sizeof(ty_wav_id_header_s))
            {
                PR_ERR("error wav file: wrong FMT size %d", id_header.size);
                ty_media_wav_close(&wav);
                return NULL;
            }
            read_len = fread(&wav->format.compression_code, 1, id_header.size, wav->fp);
            if (read_len < id_header.size)
            {
                PR_ERR("err wav file: read FMT fail");
                ty_media_wav_close(&wav);
                return NULL;
            }
            memcpy(wav->format.id, id_header.id, sizeof(wav->format.id));
            wav->format.size = id_header.size;
            wav->data_offset += sizeof(ty_wav_format_s);
        }
        else if (0 == strncasecmp("DATA", id_header.id, strlen("DATA")))
        {
            memcpy(wav->data.id, id_header.id, sizeof(wav->data.id));
            wav->data.size = id_header.size;
            wav->data_offset += sizeof(ty_wav_id_header_s);
            wav->data_size = id_header.size;
            break;
        }
        else
        {
            memcpy(id_str, id_header.id, sizeof(id_header.id));
            PR_DEBUG("unhandled chunk: %s, size: %d", id_str, id_header.size);
            wav->data_offset += sizeof(ty_wav_id_header_s) + id_header.size;
            fseek(wav->fp, id_header.size, SEEK_CUR);
        }
    }

    return wav;
}

VOID ty_media_wav_close(ty_media_wav_s **wav)
{
    ty_media_wav_s *tmp_wav = NULL;

    if (NULL == wav)
    {
        return;
    }

    tmp_wav = *wav;
    if (NULL == tmp_wav)
    {
        return;
    }

    if (NULL != tmp_wav->fp)
    {
        fclose(tmp_wav->fp);
    }

    free(tmp_wav);
    *wav = NULL;

    return;
}

VOID ty_media_wav_dump(ty_media_wav_s *wav)
{
    CHAR_T str[5] = {0};

    PR_INFO("file length: %d\n", wav->file_size);

    PR_INFO("\nRIFF WAVE Chunk\n");
    memcpy(str, wav->riff.id, sizeof(wav->riff.id));
    PR_INFO("id: %s\n", str);
    PR_INFO("size: %d\n", wav->riff.size);
    memcpy(str, wav->riff.type, sizeof(wav->riff.type));
    PR_INFO("type: %s\n", str);

    PR_INFO("\nFORMAT Chunk\n");
    memcpy(str, wav->format.id, sizeof(wav->format.id));
    PR_INFO("id: %s\n", str);
    PR_INFO("formattag: 0x%02x\n", wav->format.compression_code);
    PR_INFO("channels: %d\n", wav->format.channels);
    PR_INFO("samples_per_sec: %d\n", wav->format.samples_per_sec);
    PR_INFO("avg_bytes_per_sec: %d\n", wav->format.avg_bytes_per_sec);
    PR_INFO("block_align: %d\n", wav->format.block_align);
    PR_INFO("bits_per_sample: %d\n", wav->format.bits_per_sample);

    PR_INFO("\nDATA Chunk\n");
    memcpy(str, wav->data.id, sizeof(wav->data.id));
    PR_INFO("id: %s\n", str);
    PR_INFO("size: %d\n", wav->data.size);
    PR_INFO("data offset: %d\n", wav->data_offset);

    return;
}

INT_T ty_media_wav_check(CHAR_T *file_name)
{
    ty_media_wav_s *pwav = NULL;
    UINT_T file_len = 0;

    pwav = ty_media_wav_open(file_name);
    if (pwav == NULL)
    {
        PR_ERR("open media wav %s failed", file_name);
        return -1;
    }

    /* get file length */
    fseek(pwav->fp, 0, SEEK_END);
    file_len = ftell(pwav->fp);

    if (file_len < pwav->data_offset + pwav->data_size)
    {
        PR_ERR("file_len = %d, expected len = %d\n", file_len, pwav->data_offset + pwav->data_size);
        ty_media_wav_close(&pwav);
        return -1;
    }

    ty_media_wav_close(&pwav);
    return 0;
}
#endif
